@@ -6,7 +6,7 @@
  * Generate and verify signatures.
  */
 
-#include <header.h>
+#include <rpm/header.h>
 
 /** \ingroup signature
  * Signature types stored in rpm lead.
@@ -34,8 +34,7 @@ extern "C" {
  * Return new, empty (signature) header instance.
  * @return		signature header
  */
-Header rpmNewSignature(void)
-	/*@*/;
+Header rpmNewSignature(void);
 
 /** \ingroup signature
  * Read (and verify header+payload size) signature header.
@@ -46,20 +45,15 @@ Header rpmNewSignature(void)
  * @retval msg		failure msg
  * @return		rpmRC return code
  */
-rpmRC rpmReadSignature(FD_t fd, /*@null@*/ /*@out@*/ Header *sighp,
-		sigType sig_type, /*@null@*/ /*@out@*/ const char ** msg)
-	/*@globals fileSystem @*/
-	/*@modifies fd, *sighp, *msg, fileSystem @*/;
+rpmRC rpmReadSignature(FD_t fd, Header *sighp, sigType sig_type, char ** msg);
 
 /** \ingroup signature
  * Write signature header.
  * @param fd		file handle
- * @param sigh		(signature) header
+ * @param h		(signature) header
  * @return		0 on success, 1 on error
  */
-int rpmWriteSignature(FD_t fd, Header sigh)
-	/*@globals fileSystem @*/
-	/*@modifies fd, sigh, fileSystem @*/;
+int rpmWriteSignature(FD_t fd, Header h);
 
 /** \ingroup signature
  * Generate signature(s) from a header+payload file, save in signature header.
@@ -70,9 +64,26 @@ int rpmWriteSignature(FD_t fd, Header sigh)
  * @return		0 on success, -1 on failure
  */
 int rpmAddSignature(Header sigh, const char * file,
-		    int_32 sigTag, /*@null@*/ const char * passPhrase)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies sigh, sigTag, rpmGlobalMacroContext, fileSystem, internalState @*/;
+		    rpmSigTag sigTag, const char * passPhrase);
+
+/** \ingroup signature
+ * Verify a signature from a package.
+ *
+ * @param keyring	keyring handle
+ * @param sigtd		signature tag data container
+ * @param dig		signature/pubkey parameters
+ * @retval result	detailed text result of signature verification
+ * 			(malloc'd)
+ * @return		result of signature verification
+ */
+rpmRC rpmVerifySignature(rpmKeyring keyring, rpmtd sigtd, pgpDig dig, char ** result);
+
+/** \ingroup signature
+ * Destroy signature header from package.
+ * @param h		signature header
+ * @return		NULL always
+ */
+Header rpmFreeSignature(Header h);
 
 /******************************************************************/
 
@@ -88,9 +99,7 @@ int rpmAddSignature(Header sigh, const char * file,
  * @param action	enable/disable/query action
  * @return		sigTag to use, 0 if none, -1 on error
  */
-int rpmLookupSignatureType(int action)
-	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
-	/*@modifies rpmGlobalMacroContext, internalState @*/;
+int rpmLookupSignatureType(int action);
 
 /** \ingroup signature
  * Read a pass phrase using getpass(3), confirm with gpg/pgp helper binaries.
@@ -98,23 +107,15 @@ int rpmLookupSignatureType(int action)
  * @param sigTag	signature type/tag
  * @return		pass phrase
  */
-/*@dependent@*/ /*@null@*/
-char * rpmGetPassPhrase(/*@null@*/ const char * prompt,
-		const int sigTag)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/;
+char * rpmGetPassPhrase(const char * prompt, const rpmSigTag sigTag);
 
 /** \ingroup signature
  * Return path to pgp executable of given type, or NULL when not found.
  * @retval pgpVer	pgp version
  * @return		path to pgp executable
  */
-/*@-exportlocal -redecl@*/
-/*@null@*/ const char * rpmDetectPGPVersion(
-			/*@null@*/ /*@out@*/ pgpVersion * pgpVer)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies *pgpVer, rpmGlobalMacroContext @*/;
-/*@=exportlocal =redecl@*/
+const char * rpmDetectPGPVersion(
+			pgpVersion * pgpVer);
 
 #ifdef __cplusplus
 }
