@@ -4,7 +4,7 @@
 
 #include "system.h"
 
-#include <rpmlib.h>
+#include <rpm/header.h>
 
 #include "rpmdb-py.h"
 #include "rpmmi-py.h"
@@ -12,11 +12,10 @@
 
 #include "debug.h"
 
-/*@access Header @*/
 
 /** \ingroup python
  * \class Rpmdb
- * \brief A python rpm.db object represents an RPM database.
+ * \brief A python rpmdb object represents an RPM database.
  *
  * Instances of the rpmdb object provide access to the records of a
  * RPM database.  The records are accessed by index number.  To
@@ -99,22 +98,18 @@
  */
 
 /** \ingroup python
- * \name Class: Rpmdb
+ * \name Class: rpm.db
  */
-/*@{*/
 
 /**
  */
-/*@null@*/
 static rpmmiObject *
 rpmdb_Match (rpmdbObject * s, PyObject * args, PyObject * kwds)
-	/*@globals rpmGlobalMacroContext @*/
-	/*@modifies s, rpmGlobalMacroContext @*/
 {
     PyObject *TagN = NULL;
     char *key = NULL;
     int len = 0;
-    int tag = RPMDBI_PACKAGES;
+    rpmTag tag = RPMDBI_PACKAGES;
     char * kwlist[] = {"tagNumber", "key", "len", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|Ozi", kwlist,
@@ -126,29 +121,22 @@ rpmdb_Match (rpmdbObject * s, PyObject * args, PyObject * kwds)
 	return NULL;
     }
 
-    return rpmmi_Wrap( rpmdbInitIterator(s->db, tag, key, len) );
+    return rpmmi_Wrap( rpmdbInitIterator(s->db, tag, key, len), (PyObject *)s);
 }
 
-/*@}*/
-
 /**
  */
-/*@-fullinitblock@*/
-/*@unchecked@*/ /*@observer@*/
 static struct PyMethodDef rpmdb_methods[] = {
     {"match",	    (PyCFunction) rpmdb_Match,	METH_VARARGS|METH_KEYWORDS,
 "db.match([TagN, [key, [len]]]) -> mi\n\
 - Create an rpm db match iterator.\n" },
     {NULL,		NULL}		/* sentinel */
 };
-/*@=fullinitblock@*/
 
 /**
  */
 static int
 rpmdb_length(rpmdbObject * s)
-	/*@globals rpmGlobalMacroContext @*/
-	/*@modifies s, rpmGlobalMacroContext @*/
 {
     rpmdbMatchIterator mi;
     int count = 0;
@@ -163,11 +151,8 @@ rpmdb_length(rpmdbObject * s)
 
 /**
  */
-/*@null@*/
 static hdrObject *
 rpmdb_subscript(rpmdbObject * s, PyObject * key)
-	/*@globals rpmGlobalMacroContext @*/
-	/*@modifies s, rpmGlobalMacroContext @*/
 {
     int offset;
     hdrObject * ho;
@@ -196,9 +181,8 @@ rpmdb_subscript(rpmdbObject * s, PyObject * key)
 
 /**
  */
-/*@unchecked@*/ /*@observer@*/
 static PyMappingMethods rpmdb_as_mapping = {
-	(inquiry) rpmdb_length,		/* mp_length */
+	(lenfunc) rpmdb_length,		/* mp_length */
 	(binaryfunc) rpmdb_subscript,	/* mp_subscript */
 	(objobjargproc)0,		/* mp_ass_subscript */
 };
@@ -206,7 +190,6 @@ static PyMappingMethods rpmdb_as_mapping = {
 /**
  */
 static void rpmdb_dealloc(rpmdbObject * s)
-	/*@modifies s @*/
 {
     if (s->db != NULL)
 	rpmdbClose(s->db);
@@ -214,26 +197,22 @@ static void rpmdb_dealloc(rpmdbObject * s)
 }
 
 static PyObject * rpmdb_getattro(PyObject * o, PyObject * n)
-	/*@*/
 {
     return PyObject_GenericGetAttr(o, n);
 }
 
 static int rpmdb_setattro(PyObject * o, PyObject * n, PyObject * v)
-	/*@*/
 {
     return PyObject_GenericSetAttr(o, n, v);
 }
 
 /**
  */
-/*@unchecked@*/ /*@observer@*/
 static char rpmdb_doc[] =
 "";
 
 /**
  */
-/*@-fullinitblock@*/
 PyTypeObject rpmdb_Type = {
 	PyObject_HEAD_INIT(&PyType_Type)
 	0,				/* ob_size */
@@ -279,7 +258,6 @@ PyTypeObject rpmdb_Type = {
 	0,				/* tp_is_gc */
 #endif
 };
-/*@=fullinitblock@*/
 
 #ifdef  _LEGACY_BINDINGS_TOO
 rpmdb dbFromDb(rpmdbObject * db)
@@ -290,7 +268,7 @@ rpmdb dbFromDb(rpmdbObject * db)
 /**
  */
 rpmdbObject *
-rpmOpenDB(/*@unused@*/ PyObject * self, PyObject * args, PyObject * kwds) {
+rpmOpenDB(PyObject * self, PyObject * args, PyObject * kwds) {
     rpmdbObject * o;
     char * root = "";
     int forWrite = 0;
@@ -310,12 +288,9 @@ rpmOpenDB(/*@unused@*/ PyObject * self, PyObject * args, PyObject * kwds) {
 
 	Py_DECREF(o);
 	/* PyErr_SetString should take varargs... */
-	errsize = strlen(errmsg) + *root == '\0' ? 15 /* "/var/lib/rpm" */ : strlen(root);
-	errstr = alloca(errsize);
-/*@-formatconst@*/
-	snprintf(errstr, errsize, errmsg, *root == '\0' ? "/var/lib/rpm" : root);
-/*@=formatconst@*/
+	rasprintf(&errstr, errmsg, *root == '\0' ? "/var/lib/rpm" : root);
 	PyErr_SetString(pyrpmError, errstr);
+	free(errstr);
 	return NULL;
     }
 
@@ -326,7 +301,7 @@ rpmOpenDB(/*@unused@*/ PyObject * self, PyObject * args, PyObject * kwds) {
  * @todo Permit header checks when doing --rebuilddb.
  */
 PyObject *
-rebuildDB (/*@unused@*/ PyObject * self, PyObject * args, PyObject * kwds)
+rebuildDB (PyObject * self, PyObject * args, PyObject * kwds)
 {
     char * rootDir = "";
     char * kwlist[] = {"rootdir", NULL};
@@ -337,3 +312,4 @@ rebuildDB (/*@unused@*/ PyObject * self, PyObject * args, PyObject * kwds)
     return Py_BuildValue("i", rpmdbRebuild(rootDir, NULL, NULL));
 }
 #endif
+
