@@ -4,14 +4,15 @@
 
 #include "system.h"
 
-#include <rpmlib.h>
+#include <rpm/rpmtypes.h>
+#include <rpm/rpmstring.h>
+#include <rpm/rpmlib.h>		/* rpmvercmp */
 
 #include "header-py.h"
 #include "rpmds-py.h"
 
 #include "debug.h"
 
-/*@access rpmds @*/
 
 /**
  * Split EVR into epoch, version, and release components.
@@ -22,11 +23,9 @@
  */
 static
 void rpmds_ParseEVR(char * evr,
-		/*@exposed@*/ /*@out@*/ const char ** ep,
-		/*@exposed@*/ /*@out@*/ const char ** vp,
-		/*@exposed@*/ /*@out@*/ const char ** rp)
-	/*@modifies *ep, *vp, *rp @*/
-	/*@requires maxSet(ep) >= 0 /\ maxSet(vp) >= 0 /\ maxSet(rp) >= 0 @*/
+		const char ** ep,
+		const char ** vp,
+		const char ** rp)
 {
     const char *epoch;
     const char *version;		/* assume only version is present */
@@ -34,24 +33,20 @@ void rpmds_ParseEVR(char * evr,
     char *s, *se;
 
     s = evr;
-    while (*s && xisdigit(*s)) s++;	/* s points to epoch terminator */
+    while (*s && risdigit(*s)) s++;	/* s points to epoch terminator */
     se = strrchr(s, '-');		/* se points to version terminator */
 
     if (*s == ':') {
 	epoch = evr;
 	*s++ = '\0';
 	version = s;
-	/*@-branchstate@*/
 	if (*epoch == '\0') epoch = "0";
-	/*@=branchstate@*/
     } else {
 	epoch = NULL;	/* XXX disable epoch compare if missing */
 	version = evr;
     }
     if (se) {
-/*@-boundswrite@*/
 	*se++ = '\0';
-/*@=boundswrite@*/
 	release = se;
     } else {
 	release = NULL;
@@ -62,6 +57,78 @@ void rpmds_ParseEVR(char * evr,
     if (rp) *rp = release;
 }
 
+static PyObject *
+rpmds_Debug(rpmdsObject * s, PyObject * args, PyObject * kwds)
+{
+    char * kwlist[] = {"debugLevel", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist, &_rpmds_debug))
+	return NULL;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+rpmds_Count(rpmdsObject * s)
+{
+    return Py_BuildValue("i", rpmdsCount(s->ds));
+}
+
+static PyObject *
+rpmds_Ix(rpmdsObject * s)
+{
+    return Py_BuildValue("i", rpmdsIx(s->ds));
+}
+
+static PyObject *
+rpmds_DNEVR(rpmdsObject * s)
+{
+    return Py_BuildValue("s", rpmdsDNEVR(s->ds));
+}
+
+static PyObject *
+rpmds_N(rpmdsObject * s)
+{
+    return Py_BuildValue("s", rpmdsN(s->ds));
+}
+
+static PyObject *
+rpmds_EVR(rpmdsObject * s)
+{
+    return Py_BuildValue("s", rpmdsEVR(s->ds));
+}
+
+static PyObject *
+rpmds_Flags(rpmdsObject * s)
+{
+    return Py_BuildValue("i", rpmdsFlags(s->ds));
+}
+
+static PyObject *
+rpmds_BT(rpmdsObject * s)
+{
+    return Py_BuildValue("i", (int) rpmdsBT(s->ds));
+}
+
+static PyObject *
+rpmds_TagN(rpmdsObject * s)
+{
+    return Py_BuildValue("i", rpmdsTagN(s->ds));
+}
+
+static PyObject *
+rpmds_Color(rpmdsObject * s)
+{
+    return Py_BuildValue("i", rpmdsColor(s->ds));
+}
+
+static PyObject *
+rpmds_Refs(rpmdsObject * s)
+{
+    return Py_BuildValue("i", rpmdsRefs(s->ds));
+}
+
 /**
  */
 static int compare_values(const char *str1, const char *str2)
@@ -77,7 +144,6 @@ static int compare_values(const char *str1, const char *str2)
 
 static int
 rpmds_compare(rpmdsObject * a, rpmdsObject * b)
-	/*@*/
 {
     char *aEVR = xstrdup(rpmdsEVR(a->ds));
     const char *aE, *aV, *aR;
@@ -104,7 +170,6 @@ rpmds_compare(rpmdsObject * a, rpmdsObject * b)
 
 static PyObject *
 rpmds_richcompare(rpmdsObject * a, rpmdsObject * b, int op)
-	/*@*/
 {
     int rc;
 
@@ -119,7 +184,6 @@ rpmds_richcompare(rpmdsObject * a, rpmdsObject * b, int op)
     case Py_GT:
     case Py_GE:
     case Py_EQ:
-	/*@fallthrough@*/
     default:
 	rc = -1;
 	break;
@@ -129,16 +193,13 @@ rpmds_richcompare(rpmdsObject * a, rpmdsObject * b, int op)
 
 static PyObject *
 rpmds_iter(rpmdsObject * s)
-	/*@*/
 {
     Py_INCREF(s);
     return (PyObject *)s;
 }
 
-/*@null@*/
 static PyObject *
 rpmds_iternext(rpmdsObject * s)
-	/*@modifies s @*/
 {
     PyObject * result = NULL;
 
@@ -152,30 +213,20 @@ rpmds_iternext(rpmdsObject * s)
     if (rpmdsNext(s->ds) >= 0) {
 	const char * N = rpmdsN(s->ds);
 	const char * EVR = rpmdsEVR(s->ds);
-	int tagN = rpmdsTagN(s->ds);
-	int Flags = rpmdsFlags(s->ds);
+	rpmTag tagN = rpmdsTagN(s->ds);
+	rpmsenseFlags Flags = rpmdsFlags(s->ds);
 
-/*@-branchstate@*/
 	if (N != NULL) N = xstrdup(N);
 	if (EVR != NULL) EVR = xstrdup(EVR);
-/*@=branchstate@*/
-	result = (PyObject *)rpmds_Wrap( rpmdsSingle(tagN, N, EVR, Flags) );
+	result = (PyObject *) rpmds_Wrap( rpmdsSingle(tagN, N, EVR, Flags) );
     } else
 	s->active = 0;
 
     return result;
 }
 
-/** \ingroup python
- * \name Class: Rpmds
- */
-/*@{*/
-
-/*@null@*/
 static PyObject *
 rpmds_Next(rpmdsObject * s)
-	/*@globals _Py_NoneStruct @*/
-	/*@modifies s, _Py_NoneStruct @*/
 {
     PyObject * result;
 
@@ -188,112 +239,8 @@ rpmds_Next(rpmdsObject * s)
     return result;
 }
 
-/*@null@*/
-static PyObject *
-rpmds_Debug(/*@unused@*/ rpmdsObject * s, PyObject * args, PyObject * kwds)
-	/*@globals _Py_NoneStruct @*/
-	/*@modifies _Py_NoneStruct @*/
-{
-    char * kwlist[] = {"debugLevel", NULL};
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist, &_rpmds_debug))
-	return NULL;
-
-    Py_INCREF(Py_None);
-    return Py_None;
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Count(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsCount(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Ix(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsIx(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_DNEVR(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("s", rpmdsDNEVR(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_N(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("s", rpmdsN(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_EVR(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("s", rpmdsEVR(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Flags(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsFlags(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_BT(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", (int) rpmdsBT(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_TagN(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsTagN(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Color(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsColor(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Refs(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsRefs(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Result(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsResult(s->ds));
-}
-/*@null@*/
 static PyObject *
 rpmds_SetNoPromote(rpmdsObject * s, PyObject * args, PyObject * kwds)
-	/*@modifies s @*/
 {
     int nopromote;
     char * kwlist[] = {"noPromote", NULL};
@@ -305,11 +252,8 @@ rpmds_SetNoPromote(rpmdsObject * s, PyObject * args, PyObject * kwds)
     return Py_BuildValue("i", rpmdsSetNoPromote(s->ds, nopromote));
 }
 
-/*@null@*/
 static PyObject *
 rpmds_Notify(rpmdsObject * s, PyObject * args, PyObject * kwds)
-	/*@globals _Py_NoneStruct @*/
-	/*@modifies _Py_NoneStruct @*/
 {
     const char * where;
     int rc;
@@ -325,29 +269,20 @@ rpmds_Notify(rpmdsObject * s, PyObject * args, PyObject * kwds)
 }
 
 /* XXX rpmdsFind uses bsearch on s->ds, so a sort is needed. */
-/*@null@*/
 static PyObject *
 rpmds_Sort(rpmdsObject * s)
-	/*@globals _Py_NoneStruct @*/
-	/*@modifies _Py_NoneStruct @*/
 {
-    rpmds nds = NULL;
-
-    if (rpmdsMerge(&nds, s->ds) >= 0) {
-	s->ds = rpmdsFree(s->ds);
-	s->ds = nds;
-    }
+    /* XXX sort on (N,EVR,F) here. */
     Py_INCREF(Py_None);
     return Py_None;
 }
 
-/*@null@*/
 static PyObject *
 rpmds_Find(rpmdsObject * s, PyObject * args, PyObject * kwds)
-	/*@modifies s @*/
 {
     PyObject * to = NULL;
     rpmdsObject * o;
+    int rc;
     char * kwlist[] = {"element", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:Find", kwlist, &to))
@@ -359,13 +294,12 @@ rpmds_Find(rpmdsObject * s, PyObject * args, PyObject * kwds)
     /* XXX make sure ods index is valid, real fix in lib/rpmds.c. */
     if (rpmdsIx(o->ds) == -1)	rpmdsSetIx(o->ds, 0);
 
-    return Py_BuildValue("i", rpmdsFind(s->ds, o->ds));
+    rc = rpmdsFind(s->ds, o->ds);
+    return Py_BuildValue("i", rc);
 }
 
-/*@null@*/
 static PyObject *
 rpmds_Merge(rpmdsObject * s, PyObject * args, PyObject * kwds)
-	/*@modifies s @*/
 {
     PyObject * to = NULL;
     rpmdsObject * o;
@@ -378,40 +312,23 @@ rpmds_Merge(rpmdsObject * s, PyObject * args, PyObject * kwds)
     o = (rpmdsObject *)to;
     return Py_BuildValue("i", rpmdsMerge(&s->ds, o->ds));
 }
-
-/*@null@*/
 static PyObject *
 rpmds_Search(rpmdsObject * s, PyObject * args, PyObject * kwds)
-	/*@modifies s @*/
 {
     PyObject * to = NULL;
     rpmdsObject * o;
     char * kwlist[] = {"element", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:Merge", kwlist, &to))
-	return NULL;
+        return NULL;
 
     /* XXX ds type check needed. */
     o = (rpmdsObject *)to;
     return Py_BuildValue("i", rpmdsSearch(s->ds, o->ds));
 }
 
-static PyObject *
-rpmds_Cpuinfo(rpmdsObject * s)
-	/*@*/
-{
-    rpmds ds = NULL;
-    int xx;
-
-    /* XXX check return code, permit arg (NULL uses system default). */
-    xx = rpmdsCpuinfo(&ds, NULL);
-
-    return (PyObject *) rpmds_Wrap( ds );
-}
-
 static PyObject *
 rpmds_Rpmlib(rpmdsObject * s)
-	/*@*/
 {
     rpmds ds = NULL;
     int xx;
@@ -422,55 +339,10 @@ rpmds_Rpmlib(rpmdsObject * s)
     return (PyObject *) rpmds_Wrap( ds );
 }
 
-static PyObject *
-rpmds_Sysinfo(rpmdsObject * s)
-	/*@*/
-{
-    rpmPRCO PRCO = rpmdsNewPRCO(NULL);
-    rpmds P = NULL;
-    int xx;
-
-    /* XXX check return code, permit arg (NULL uses system default). */
-    xx = rpmdsSysinfo(PRCO, NULL);
-    P = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME), __FUNCTION__);
-    PRCO = rpmdsFreePRCO(PRCO);
-
-    return (PyObject *) rpmds_Wrap( P );
-}
-
-static PyObject *
-rpmds_Getconf(rpmdsObject * s)
-	/*@*/
-{
-    rpmds ds = NULL;
-    int xx;
-
-    /* XXX check return code, permit arg (NULL uses system default). */
-    xx = rpmdsGetconf(&ds, NULL);
-
-    return (PyObject *) rpmds_Wrap( ds );
-}
-
-static PyObject *
-rpmds_Ldconfig(rpmdsObject * s)
-	/*@*/
-{
-    rpmPRCO PRCO = rpmdsNewPRCO(NULL);
-    rpmds P = NULL;
-    int xx;
-
-    /* XXX check return code, permit arg (NULL uses system default). */
-    xx = rpmdsLdconfig(PRCO, NULL);
-
-    P = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME), __FUNCTION__);
-    PRCO = rpmdsFreePRCO(PRCO);
-    return (PyObject *) rpmds_Wrap( P );
-}
 
 #ifdef	NOTYET
 static PyObject *
 rpmds_Compare(rpmdsObject * s, PyObject * args, PyObject * kwds)
-	/*@modifies s @*/
 {
     PyObject * to = NULL;
     rpmdsObject * o;
@@ -484,10 +356,8 @@ rpmds_Compare(rpmdsObject * s, PyObject * args, PyObject * kwds)
     return Py_BuildValue("i", rpmdsCompare(s->ds, o->ds));
 }
 
-/*@null@*/
 static PyObject *
 rpmds_Problem(rpmdsObject * s)
-	/*@*/
 {
     if (!PyArg_ParseTuple(args, ":Problem"))
 	return NULL;
@@ -496,10 +366,6 @@ rpmds_Problem(rpmdsObject * s)
 }
 #endif
 
-/*@}*/
-
-/*@-fullinitblock@*/
-/*@unchecked@*/ /*@observer@*/
 static struct PyMethodDef rpmds_methods[] = {
  {"Debug",	(PyCFunction)rpmds_Debug,	METH_VARARGS|METH_KEYWORDS,
 	NULL},
@@ -523,8 +389,6 @@ static struct PyMethodDef rpmds_methods[] = {
 	"ds.Color -> Color	- Return current Color.\n" },
  {"Refs",	(PyCFunction)rpmds_Refs,	METH_NOARGS,
 	"ds.Refs -> Refs	- Return current Refs.\n" },
- {"Result",	(PyCFunction)rpmds_Result,	METH_NOARGS,
-	"ds.Result -> Result	- Return current Result.\n" },
  {"next",	(PyCFunction)rpmds_Next,	METH_NOARGS,
 "ds.next() -> (N, EVR, Flags)\n\
 - Retrieve next dependency triple.\n" },
@@ -533,29 +397,17 @@ static struct PyMethodDef rpmds_methods[] = {
  {"Notify",	(PyCFunction)rpmds_Notify,	METH_VARARGS|METH_KEYWORDS,
 	NULL},
  {"Sort",	(PyCFunction)rpmds_Sort,	METH_NOARGS,
-"ds.Sort() -> None\n\
-- Sort the (N,EVR,Flags) elements in ds\n" },
+	NULL},
  {"Find",	(PyCFunction)rpmds_Find,	METH_VARARGS|METH_KEYWORDS,
-"ds.Find(element) -> matching ds index (-1 on failure)\n\
-- Check for an exactly matching element in ds.\n\
-The current index in ds is positioned at matching member upon success.\n" },
+	NULL},
  {"Merge",	(PyCFunction)rpmds_Merge,	METH_VARARGS|METH_KEYWORDS,
-"ds.Merge(elements) -> 0 on success\n\
-- Merge elements into ds, maintaining (N,EVR,Flags) sort order.\n" },
- {"Search",	(PyCFunction)rpmds_Search,	METH_VARARGS|METH_KEYWORDS,
+	NULL},
+ {"Search",     (PyCFunction)rpmds_Search,      METH_VARARGS|METH_KEYWORDS,
 "ds.Search(element) -> matching ds index (-1 on failure)\n\
 - Check that element dependency range overlaps some member of ds.\n\
 The current index in ds is positioned at overlapping member upon success.\n" },
- {"Cpuinfo",	(PyCFunction)rpmds_Cpuinfo,	METH_NOARGS|METH_STATIC,
-	"ds.Cpuinfo -> nds	- Return /proc/cpuinfo dependency set.\n"},
- {"Rpmlib",	(PyCFunction)rpmds_Rpmlib,	METH_NOARGS|METH_STATIC,
-	"ds.Rpmlib -> nds	- Return internal rpmlib dependency set.\n"},
- {"Sysinfo",	(PyCFunction)rpmds_Sysinfo,	METH_NOARGS|METH_STATIC,
-	"ds.Sysinfo -> nds	- Return /etc/rpm/sysinfo dependency set.\n"},
- {"Getconf",	(PyCFunction)rpmds_Getconf,	METH_NOARGS|METH_STATIC,
-	"ds.Getconf -> nds	- Return getconf(1) dependency set.\n"},
- {"Ldconfig",	(PyCFunction)rpmds_Ldconfig,	METH_NOARGS|METH_STATIC,
-	"ds.Ldconfig -> nds	- Return /etc/ld.so.cache dependency set.\n"},
+ {"Rpmlib",     (PyCFunction)rpmds_Rpmlib,      METH_NOARGS|METH_STATIC,
+	"ds.Rpmlib -> nds       - Return internal rpmlib dependency set.\n"},
 #ifdef	NOTYET
  {"Compare",	(PyCFunction)rpmds_Compare,	METH_VARARGS|METH_KEYWORDS,
 	NULL},
@@ -564,13 +416,11 @@ The current index in ds is positioned at overlapping member upon success.\n" },
 #endif
  {NULL,		NULL}		/* sentinel */
 };
-/*@=fullinitblock@*/
 
 /* ---------- */
 
 static void
 rpmds_dealloc(rpmdsObject * s)
-	/*@modifies s @*/
 {
     if (s) {
 	s->ds = rpmdsFree(s->ds);
@@ -579,9 +429,7 @@ rpmds_dealloc(rpmdsObject * s)
 }
 
 static int
-rpmds_print(rpmdsObject * s, FILE * fp, /*@unused@*/ int flags)
-	/*@globals fileSystem @*/
-	/*@modifies s, fp, fileSystem @*/
+rpmds_print(rpmdsObject * s, FILE * fp, int flags)
 {
     if (!(s && s->ds))
 	return -1;
@@ -593,28 +441,23 @@ rpmds_print(rpmdsObject * s, FILE * fp, /*@unused@*/ int flags)
 }
 
 static PyObject * rpmds_getattro(PyObject * o, PyObject * n)
-	/*@*/
 {
     return PyObject_GenericGetAttr(o, n);
 }
 
 static int rpmds_setattro(PyObject * o, PyObject * n, PyObject * v)
-	/*@*/
 {
     return PyObject_GenericSetAttr(o, n, v);
 }
 
 static int
 rpmds_length(rpmdsObject * s)
-	/*@*/
 {
     return rpmdsCount(s->ds);
 }
 
-/*@null@*/
 static PyObject *
 rpmds_subscript(rpmdsObject * s, PyObject * key)
-	/*@modifies s @*/
 {
     int ix;
 
@@ -631,21 +474,19 @@ rpmds_subscript(rpmdsObject * s, PyObject * key)
 }
 
 static PyMappingMethods rpmds_as_mapping = {
-        (inquiry) rpmds_length,		/* mp_length */
+        (lenfunc) rpmds_length,		/* mp_length */
         (binaryfunc) rpmds_subscript,	/* mp_subscript */
         (objobjargproc)0,		/* mp_ass_subscript */
 };
 
-/**
+/** \ingroup py_c
  */
 static int rpmds_init(rpmdsObject * s, PyObject *args, PyObject *kwds)
-	/*@globals rpmGlobalMacroContext @*/
-	/*@modifies s, rpmGlobalMacroContext @*/
 {
     hdrObject * ho = NULL;
     PyObject * to = NULL;
-    int tagN = RPMTAG_REQUIRENAME;
-    int flags = 0;
+    rpmTag tagN = RPMTAG_REQUIRENAME;
+    rpmsenseFlags flags = 0;
     char * kwlist[] = {"header", "tag", "flags", NULL};
 
 if (_rpmds_debug < 0)
@@ -662,16 +503,15 @@ fprintf(stderr, "*** rpmds_init(%p,%p,%p)\n", s, args, kwds);
 	    return -1;
 	}
     }
-    s->ds = rpmdsNew(hdrGetHeader(ho), tagN, flags);
+    s->ds = rpmdsNew(hdrGetHeader(ho), tagN, 0);
     s->active = 0;
 
     return 0;
 }
 
-/**
+/** \ingroup py_c
  */
-static void rpmds_free(/*@only@*/ rpmdsObject * s)
-	/*@modifies s @*/
+static void rpmds_free(rpmdsObject * s)
 {
 if (_rpmds_debug)
 fprintf(stderr, "%p -- ds %p\n", s, s->ds);
@@ -680,10 +520,9 @@ fprintf(stderr, "%p -- ds %p\n", s, s->ds);
     PyObject_Del((PyObject *)s);
 }
 
-/**
+/** \ingroup py_c
  */
 static PyObject * rpmds_alloc(PyTypeObject * subtype, int nitems)
-	/*@*/
 {
     PyObject * s = PyType_GenericAlloc(subtype, nitems);
 
@@ -692,12 +531,9 @@ fprintf(stderr, "*** rpmds_alloc(%p,%d) ret %p\n", subtype, nitems, s);
     return s;
 }
 
-/**
+/** \ingroup py_c
  */
-/*@null@*/
 static PyObject * rpmds_new(PyTypeObject * subtype, PyObject *args, PyObject *kwds)
-	/*@globals rpmGlobalMacroContext @*/
-	/*@modifies rpmGlobalMacroContext @*/
 {
     rpmdsObject * s = (void *) PyObject_New(rpmdsObject, subtype);
 
@@ -715,11 +551,9 @@ fprintf(stderr, "%p ++ ds %p\n", s, s->ds);
 
 /**
  */
-/*@unchecked@*/ /*@observer@*/
 static char rpmds_doc[] =
 "";
 
-/*@-fullinitblock@*/
 PyTypeObject rpmds_Type = {
 	PyObject_HEAD_INIT(&PyType_Type)
 	0,				/* ob_size */
@@ -763,11 +597,10 @@ PyTypeObject rpmds_Type = {
 	(initproc) rpmds_init,		/* tp_init */
 	(allocfunc) rpmds_alloc,	/* tp_alloc */
 	(newfunc) rpmds_new,		/* tp_new */
-	rpmds_free,			/* tp_free */
+	(freefunc) rpmds_free,		/* tp_free */
 	0,				/* tp_is_gc */
 #endif
 };
-/*@=fullinitblock@*/
 
 /* ---------- */
 
@@ -788,15 +621,14 @@ rpmds_Wrap(rpmds ds)
     return s;
 }
 
-
 rpmdsObject *
-rpmds_Single(/*@unused@*/ PyObject * s, PyObject * args, PyObject * kwds)
+rpmds_Single(PyObject * s, PyObject * args, PyObject * kwds)
 {
     PyObject * to = NULL;
-    int tagN = RPMTAG_PROVIDENAME;
+    rpmTag tagN = RPMTAG_PROVIDENAME;
     const char * N;
     const char * EVR = NULL;
-    int Flags = 0;
+    rpmsenseFlags Flags = 0;
     char * kwlist[] = {"to", "name", "evr", "flags", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "Os|si:Single", kwlist,
@@ -821,7 +653,7 @@ hdr_dsFromHeader(PyObject * s, PyObject * args, PyObject * kwds)
     hdrObject * ho = (hdrObject *)s;
     PyObject * to = NULL;
     rpmTag tagN = RPMTAG_REQUIRENAME;
-    int flags = 0;
+    rpmsenseFlags flags = 0;
     char * kwlist[] = {"to", "flags", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|Oi:dsFromHeader", kwlist,
@@ -835,15 +667,15 @@ hdr_dsFromHeader(PyObject * s, PyObject * args, PyObject * kwds)
 	    return NULL;
 	}
     }
-    return rpmds_Wrap( rpmdsNew(hdrGetHeader(ho), tagN, flags) );
+    return rpmds_Wrap( rpmdsNew(hdrGetHeader(ho), tagN, 0) );
 }
 
 rpmdsObject *
 hdr_dsOfHeader(PyObject * s)
 {
     hdrObject * ho = (hdrObject *)s;
-    int tagN = RPMTAG_PROVIDENAME;
-    int Flags = RPMSENSE_EQUAL;
+    rpmTag tagN = RPMTAG_PROVIDENAME;
+    rpmsenseFlags Flags = RPMSENSE_EQUAL;
 
     return rpmds_Wrap( rpmdsThis(hdrGetHeader(ho), tagN, Flags) );
 }
