@@ -4,38 +4,29 @@
 
 #include "system.h"
 
-#include <rpmio_internal.h>
-#include <rpmcli.h>	/* XXX for rpmCheckSig */
-#include <rpmdb.h>
-
-#include "legacy.h"
-#include "misc.h"
-#include "header_internal.h"
+#include <rpm/rpmlib.h>		/* rpmMachineScore, rpmReadConfigFiles */
+#include <rpm/rpmtag.h>
+#include <rpm/rpmdb.h>
+#include <rpm/rpmsq.h>
+#include <rpm/rpmlog.h>
 
 #include "header-py.h"
 #include "rpmal-py.h"
 #include "rpmds-py.h"
 #include "rpmfd-py.h"
-#include "rpmfts-py.h"
 #include "rpmfi-py.h"
 #include "rpmmi-py.h"
 #include "rpmps-py.h"
-#include "rpmrc-py.h"
+#include "rpmmacro-py.h"
 #include "rpmte-py.h"
 #include "rpmts-py.h"
 #include "spec-py.h"
 
 #include "debug.h"
 
-#ifdef __LCLINT__
-#undef	PyObject_HEAD
-#define	PyObject_HEAD	int _PyObjectHead
-#endif
-
 /** \ingroup python
  * \name Module: rpm
  */
-/*@{*/
 
 /**
  */
@@ -57,18 +48,41 @@ static PyObject * archScore(PyObject * self, PyObject * args, PyObject * kwds)
     return Py_BuildValue("i", score);
 }
 
-extern sigset_t rpmsqCaught;
 /**
- */
-static PyObject * sqCaught(PyObject * self, PyObject * args)
+ *  */
+static PyObject * signalsCaught(PyObject * self, PyObject * check)
 {
-    if (!PyArg_ParseTuple(args, ":sqCaught")) return NULL;
+    PyObject *caught, *o;
+    int llen;
+    int signum, i;
+    sigset_t newMask, oldMask;
 
-    return Py_BuildValue("i", rpmsqCaught);
+    if (!PyList_Check(check)) {
+	PyErr_SetString(PyExc_TypeError, "list expected");
+	return NULL;
+    }
+
+    llen = PyList_Size(check);
+    caught = PyList_New(0);
+
+    /* block signals while checking for them */
+    (void) sigfillset(&newMask);
+    (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
+
+    for (i = 0; i < llen; i++) {
+	o = PyList_GetItem(check, i);
+	signum = PyInt_AsLong(o);
+	if (rpmsqIsCaught(signum) > 0) {
+	    PyList_Append(caught, o);
+	}
+    }
+    (void) sigprocmask(SIG_SETMASK, &oldMask, NULL);
+
+    return caught;
 }
 
 /**
- */
+ *  */
 static PyObject * checkSignals(PyObject * self, PyObject * args)
 {
     if (!PyArg_ParseTuple(args, ":checkSignals")) return NULL;
@@ -77,6 +91,7 @@ static PyObject * checkSignals(PyObject * self, PyObject * args)
     return Py_None;
 }
 
+
 /**
  */
 static PyObject * setLogFile (PyObject * self, PyObject * args, PyObject *kwds)
@@ -146,7 +161,6 @@ static PyObject * setStats (PyObject * self, PyObject * args, PyObject * kwds)
     Py_INCREF(Py_None);
     return (PyObject *) Py_None;
 }
-/*@}*/
 
 /**
  */
@@ -155,22 +169,20 @@ static PyMethodDef rpmModuleMethods[] = {
 "rpm.TransactionSet([rootDir, [db]]) -> ts\n\
 - Create a transaction set.\n" },
 
-#if Py_TPFLAGS_HAVE_ITER        /* XXX backport to python-1.5.2 */
-    { "newrc", (PyCFunction) rpmrc_Create, METH_VARARGS|METH_KEYWORDS,
+    { "addMacro", (PyCFunction) rpmmacro_AddMacro, METH_VARARGS|METH_KEYWORDS,
 	NULL },
-#endif
-    { "addMacro", (PyCFunction) rpmrc_AddMacro, METH_VARARGS|METH_KEYWORDS,
+    { "delMacro", (PyCFunction) rpmmacro_DelMacro, METH_VARARGS|METH_KEYWORDS,
 	NULL },
-    { "delMacro", (PyCFunction) rpmrc_DelMacro, METH_VARARGS|METH_KEYWORDS,
+    { "expandMacro", (PyCFunction) rpmmacro_ExpandMacro, METH_VARARGS|METH_KEYWORDS,
 	NULL },
 
     { "archscore", (PyCFunction) archScore, METH_VARARGS|METH_KEYWORDS,
 	NULL },
 
-    { "sqCaught", (PyCFunction) sqCaught, METH_VARARGS,
+    { "signalsCaught", (PyCFunction) signalsCaught, METH_O, 
 	NULL },
     { "checkSignals", (PyCFunction) checkSignals, METH_VARARGS,
-	NULL },
+        NULL },
 
     { "headerLoad", (PyCFunction) hdrLoad, METH_VARARGS|METH_KEYWORDS,
 	NULL },
@@ -203,6 +215,14 @@ static PyMethodDef rpmModuleMethods[] = {
     { NULL }
 } ;
 
+/*
+* Force clean up of open iterators and dbs on exit.
+*/
+static void rpm_exithook(void)
+{
+   rpmdbCheckTerminate(1);
+}
+
 /**
  */
 static char rpm__doc__[] =
@@ -213,25 +233,17 @@ void init_rpm(void);	/* XXX eliminate gcc warning */
  */
 void init_rpm(void)
 {
-    PyObject * d, *o, * tag = NULL, * dict;
-    int i;
-    const struct headerSprintfExtension_s * extensions = rpmHeaderFormats;
-    struct headerSprintfExtension_s * ext;
-    PyObject * m;
+    PyObject * d, *o, *tag = NULL, *dict, *m;
 
 #if Py_TPFLAGS_HAVE_ITER        /* XXX backport to python-1.5.2 */
     if (PyType_Ready(&hdr_Type) < 0) return;
     if (PyType_Ready(&rpmal_Type) < 0) return;
     if (PyType_Ready(&rpmds_Type) < 0) return;
     if (PyType_Ready(&rpmfd_Type) < 0) return;
-    if (PyType_Ready(&rpmfts_Type) < 0) return;
     if (PyType_Ready(&rpmfi_Type) < 0) return;
     if (PyType_Ready(&rpmmi_Type) < 0) return;
     if (PyType_Ready(&rpmps_Type) < 0) return;
 
-    rpmrc_Type.tp_base = &PyDict_Type;
-    if (PyType_Ready(&rpmrc_Type) < 0) return;
-
     if (PyType_Ready(&rpmte_Type) < 0) return;
     if (PyType_Ready(&rpmts_Type) < 0) return;
     if (PyType_Ready(&spec_Type) < 0) return;
@@ -241,7 +253,14 @@ void init_rpm(void)
     if (m == NULL)
 	return;
 
-    rpmReadConfigFiles(rpmRcfiles, NULL);
+    /* 
+     * treat error to register rpm cleanup hook as fatal, tracebacks
+     * can and will leave stale locks around if we can't clean up
+     */
+    if (Py_AtExit(rpm_exithook) == -1)
+	return;
+
+    rpmReadConfigFiles(NULL, NULL);
 
     d = PyModule_GetDict(m);
 
@@ -268,9 +287,6 @@ void init_rpm(void)
     Py_INCREF(&rpmfd_Type);
     PyModule_AddObject(m, "fd", (PyObject *) &rpmfd_Type);
 
-    Py_INCREF(&rpmfts_Type);
-    PyModule_AddObject(m, "fts", (PyObject *) &rpmfts_Type);
-
     Py_INCREF(&rpmfi_Type);
     PyModule_AddObject(m, "fi", (PyObject *) &rpmfi_Type);
 
@@ -280,9 +296,6 @@ void init_rpm(void)
     Py_INCREF(&rpmps_Type);
     PyModule_AddObject(m, "ps", (PyObject *) &rpmps_Type);
 
-    Py_INCREF(&rpmrc_Type);
-    PyModule_AddObject(m, "rc", (PyObject *) &rpmrc_Type);
-
     Py_INCREF(&rpmte_Type);
     PyModule_AddObject(m, "te", (PyObject *) &rpmte_Type);
 
@@ -296,7 +309,6 @@ void init_rpm(void)
     rpmal_Type.ob_type = &PyType_Type;
     rpmds_Type.ob_type = &PyType_Type;
     rpmfd_Type.ob_type = &PyType_Type;
-    rpmfts_Type.ob_type = &PyType_Type;
     rpmfi_Type.ob_type = &PyType_Type;
     rpmmi_Type.ob_type = &PyType_Type;
     rpmps_Type.ob_type = &PyType_Type;
@@ -306,26 +318,22 @@ void init_rpm(void)
 #endif
 
     dict = PyDict_New();
+    {	const char *tname, *sname;
+	rpmtd names = rpmtdNew();
+	rpmTagGetNames(names, 1);
 
-    for (i = 0; i < rpmTagTableSize; i++) {
-	tag = PyInt_FromLong(rpmTagTable[i].val);
-	PyDict_SetItemString(d, (char *) rpmTagTable[i].name, tag);
-	Py_DECREF(tag);
-        PyDict_SetItem(dict, tag, o=PyString_FromString(rpmTagTable[i].name + 7));
-	Py_DECREF(o);
-    }
-
-    while (extensions->name) {
-	if (extensions->type == HEADER_EXT_TAG) {
-            ext = extensions;
-            PyDict_SetItemString(d, (char *) extensions->name, o=PyCObject_FromVoidPtr(ext, NULL));
+	while ((tname = rpmtdNextString(names))) {
+	    sname = tname + strlen("RPMTAG_");
+	    tag = PyInt_FromLong(rpmTagGetValue(sname));
+	    PyDict_SetItemString(d, tname, tag);
+	    Py_DECREF(tag);
+	    o = PyString_FromString(sname);
+	    PyDict_SetItem(dict, tag, o);
 	    Py_DECREF(o);
-            PyDict_SetItem(dict, tag, o=PyString_FromString(ext->name + 7));
-	    Py_DECREF(o);
-        }
-        extensions++;
-    }
-
+	}
+	rpmtdFreeData(names);
+	rpmtdFree(names);
+    }	
     PyDict_SetItemString(d, "tagnames", dict);
     Py_DECREF(dict);
 
@@ -334,6 +342,8 @@ void init_rpm(void)
     PyDict_SetItemString(d, #val, o=PyInt_FromLong( val )); \
     Py_DECREF(o);
 
+    REGISTER_ENUM(RPMTAG_NOT_FOUND);
+
     REGISTER_ENUM(RPMFILE_STATE_NORMAL);
     REGISTER_ENUM(RPMFILE_STATE_REPLACED);
     REGISTER_ENUM(RPMFILE_STATE_NOTINSTALLED);
@@ -358,18 +368,27 @@ void init_rpm(void)
     REGISTER_ENUM(RPMSENSE_LESS);
     REGISTER_ENUM(RPMSENSE_GREATER);
     REGISTER_ENUM(RPMSENSE_EQUAL);
-    REGISTER_ENUM(RPMSENSE_NOTEQUAL);
-
-    REGISTER_ENUM(RPMDEPS_FLAG_NOUPGRADE);
-    REGISTER_ENUM(RPMDEPS_FLAG_NOREQUIRES);
-    REGISTER_ENUM(RPMDEPS_FLAG_NOCONFLICTS);
-    REGISTER_ENUM(RPMDEPS_FLAG_NOOBSOLETES);
-    REGISTER_ENUM(RPMDEPS_FLAG_NOPARENTDIRS);
-    REGISTER_ENUM(RPMDEPS_FLAG_NOLINKTOS);
-    REGISTER_ENUM(RPMDEPS_FLAG_ANACONDA);
-    REGISTER_ENUM(RPMDEPS_FLAG_NOSUGGEST);
-    REGISTER_ENUM(RPMDEPS_FLAG_ADDINDEPS);
-    REGISTER_ENUM(RPMDEPS_FLAG_DEPLOOPS);
+    REGISTER_ENUM(RPMSENSE_PREREQ);
+    REGISTER_ENUM(RPMSENSE_INTERP);
+    REGISTER_ENUM(RPMSENSE_SCRIPT_PRE);
+    REGISTER_ENUM(RPMSENSE_SCRIPT_POST);
+    REGISTER_ENUM(RPMSENSE_SCRIPT_PREUN);
+    REGISTER_ENUM(RPMSENSE_SCRIPT_POSTUN);
+    REGISTER_ENUM(RPMSENSE_SCRIPT_VERIFY);
+    REGISTER_ENUM(RPMSENSE_FIND_REQUIRES);
+    REGISTER_ENUM(RPMSENSE_FIND_PROVIDES);
+    REGISTER_ENUM(RPMSENSE_TRIGGERIN);
+    REGISTER_ENUM(RPMSENSE_TRIGGERUN);
+    REGISTER_ENUM(RPMSENSE_TRIGGERPOSTUN);
+    REGISTER_ENUM(RPMSENSE_SCRIPT_PREP);
+    REGISTER_ENUM(RPMSENSE_SCRIPT_BUILD);
+    REGISTER_ENUM(RPMSENSE_SCRIPT_INSTALL);
+    REGISTER_ENUM(RPMSENSE_SCRIPT_CLEAN);
+    REGISTER_ENUM(RPMSENSE_RPMLIB);
+    REGISTER_ENUM(RPMSENSE_TRIGGERPREIN);
+    REGISTER_ENUM(RPMSENSE_KEYRING);
+    REGISTER_ENUM(RPMSENSE_PATCHES);
+    REGISTER_ENUM(RPMSENSE_CONFIG);
 
     REGISTER_ENUM(RPMTRANS_FLAG_TEST);
     REGISTER_ENUM(RPMTRANS_FLAG_BUILD_PROBS);
@@ -380,6 +399,7 @@ void init_rpm(void)
     REGISTER_ENUM(RPMTRANS_FLAG_ALLFILES);
     REGISTER_ENUM(RPMTRANS_FLAG_KEEPOBSOLETE);
     REGISTER_ENUM(RPMTRANS_FLAG_REPACKAGE);
+    REGISTER_ENUM(RPMTRANS_FLAG_REVERSE);
     REGISTER_ENUM(RPMTRANS_FLAG_NOPRE);
     REGISTER_ENUM(RPMTRANS_FLAG_NOPOST);
     REGISTER_ENUM(RPMTRANS_FLAG_NOTRIGGERPREIN);
@@ -388,7 +408,10 @@ void init_rpm(void)
     REGISTER_ENUM(RPMTRANS_FLAG_NOPREUN);
     REGISTER_ENUM(RPMTRANS_FLAG_NOPOSTUN);
     REGISTER_ENUM(RPMTRANS_FLAG_NOTRIGGERPOSTUN);
-    REGISTER_ENUM(RPMTRANS_FLAG_NOFDIGESTS);
+    REGISTER_ENUM(RPMTRANS_FLAG_NOMD5);
+    REGISTER_ENUM(RPMTRANS_FLAG_NOFILEDIGEST);
+    REGISTER_ENUM(RPMTRANS_FLAG_NOSUGGEST);
+    REGISTER_ENUM(RPMTRANS_FLAG_ADDINDEPS);
     REGISTER_ENUM(RPMTRANS_FLAG_NOCONFIGS);
 
     REGISTER_ENUM(RPMPROB_FILTER_IGNOREOS);
@@ -417,6 +440,7 @@ void init_rpm(void)
     REGISTER_ENUM(RPMCALLBACK_REPACKAGE_STOP);
     REGISTER_ENUM(RPMCALLBACK_UNPACK_ERROR);
     REGISTER_ENUM(RPMCALLBACK_CPIO_ERROR);
+    REGISTER_ENUM(RPMCALLBACK_SCRIPT_ERROR);
 
     REGISTER_ENUM(RPMPROB_BADARCH);
     REGISTER_ENUM(RPMPROB_BADOS);
@@ -429,7 +453,6 @@ void init_rpm(void)
     REGISTER_ENUM(RPMPROB_OLDPACKAGE);
     REGISTER_ENUM(RPMPROB_DISKSPACE);
     REGISTER_ENUM(RPMPROB_DISKNODES);
-    REGISTER_ENUM(RPMPROB_BADPRETRANS);
 
     REGISTER_ENUM(VERIFY_DIGEST);
     REGISTER_ENUM(VERIFY_SIGNATURE);
@@ -469,5 +492,6 @@ void init_rpm(void)
 
     REGISTER_ENUM(RPMDBI_PACKAGES);
 
-    REGISTER_ENUM((int)RPMAL_NOMATCH);
+    REGISTER_ENUM((intptr_t)RPMAL_NOMATCH);
 }
+
