@@ -4,28 +4,14 @@
 
 #include "system.h"
 
-#include "rpmio_internal.h"	/* XXX fdGetFILE() */
-#include <argv.h>
+#include <rpm/argv.h>
+#include <rpm/rpmstring.h>
 
 #include "debug.h"
 
-/*@-bounds@*/
-/**
- * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
- * @param p		memory to free
- * @return		NULL always
- */
-/*@unused@*/ static inline /*@null@*/
-void * _free(/*@only@*/ /*@null@*/ /*@out@*/ const void * p)
-	/*@modifies p @*/
+void argvPrint(const char * msg, ARGV_const_t argv, FILE * fp)
 {
-    if (p != NULL)	free((void *)p);
-    return NULL;
-}
-
-void argvPrint(const char * msg, ARGV_t argv, FILE * fp)
-{
-    ARGV_t av;
+    ARGV_const_t av;
 
     if (fp == NULL) fp = stderr;
 
@@ -38,6 +24,12 @@ void argvPrint(const char * msg, ARGV_t argv, FILE * fp)
 
 }
 
+ARGV_t argvNew(void)
+{
+    ARGV_t argv = xcalloc(1, sizeof(*argv));
+    return argv;
+}
+
 ARGI_t argiFree(ARGI_t argi)
 {
     if (argi) {
@@ -48,20 +40,18 @@ ARGI_t argiFree(ARGI_t argi)
     return NULL;
 }
 
-ARGV_t argvFree(/*@only@*/ /*@null@*/ ARGV_t argv)
+ARGV_t argvFree(ARGV_t argv)
 {
     ARGV_t av;
     
-/*@-branchstate@*/
     if (argv)
     for (av = argv; *av; av++)
 	*av = _free(*av);
-/*@=branchstate@*/
     argv = _free(argv);
     return NULL;
 }
 
-int argiCount(ARGI_t argi)
+int argiCount(ARGI_const_t argi)
 {
     int nvals = 0;
     if (argi)
@@ -69,7 +59,7 @@ int argiCount(ARGI_t argi)
     return nvals;
 }
 
-ARGint_t argiData(ARGI_t argi)
+ARGint_t argiData(ARGI_const_t argi)
 {
     ARGint_t vals = NULL;
     if (argi && argi->nvals > 0)
@@ -77,7 +67,7 @@ ARGint_t argiData(ARGI_t argi)
     return vals;
 }
 
-int argvCount(const ARGV_t argv)
+int argvCount(ARGV_const_t argv)
 {
     int argc = 0;
     if (argv)
@@ -88,17 +78,13 @@ int argvCount(const ARGV_t argv)
 
 ARGV_t argvData(ARGV_t argv)
 {
-/*@-retalias -temptrans @*/
     return argv;
-/*@=retalias =temptrans @*/
 }
 
 int argvCmp(const void * a, const void * b)
 {
-/*@-boundsread@*/
-    ARGstr_t astr = *(ARGV_t)a;
-    ARGstr_t bstr = *(ARGV_t)b;
-/*@=boundsread@*/
+    const char *astr = *(ARGV_t)a;
+    const char *bstr = *(ARGV_t)b;
     return strcmp(astr, bstr);
 }
 
@@ -110,7 +96,7 @@ int argvSort(ARGV_t argv, int (*compar)(const void *, const void *))
     return 0;
 }
 
-ARGV_t argvSearch(ARGV_t argv, ARGstr_t val,
+ARGV_t argvSearch(ARGV_const_t argv, const char *val,
 		int (*compar)(const void *, const void *))
 {
     if (argv == NULL)
@@ -120,7 +106,7 @@ ARGV_t argvSearch(ARGV_t argv, ARGstr_t val,
     return bsearch(&val, argv, argvCount(argv), sizeof(*argv), compar);
 }
 
-int argiAdd(/*@out@*/ ARGI_t * argip, int ix, int val)
+int argiAdd(ARGI_t * argip, int ix, int val)
 {
     ARGI_t argi;
 
@@ -141,7 +127,7 @@ int argiAdd(/*@out@*/ ARGI_t * argip, int ix, int val)
     return 0;
 }
 
-int argvAdd(/*@out@*/ ARGV_t * argvp, ARGstr_t val)
+int argvAdd(ARGV_t * argvp, const char *val)
 {
     ARGV_t argv;
     int argc;
@@ -149,16 +135,24 @@ int argvAdd(/*@out@*/ ARGV_t * argvp, ARGstr_t val)
     if (argvp == NULL)
 	return -1;
     argc = argvCount(*argvp);
-/*@-unqualifiedtrans@*/
     *argvp = xrealloc(*argvp, (argc + 1 + 1) * sizeof(**argvp));
-/*@=unqualifiedtrans@*/
     argv = *argvp;
     argv[argc++] = xstrdup(val);
     argv[argc  ] = NULL;
     return 0;
 }
 
-int argvAppend(/*@out@*/ ARGV_t * argvp, const ARGV_t av)
+int argvAddNum(ARGV_t *argvp, int val)
+{
+    char *valstr = NULL;
+    int rc;
+    rasprintf(&valstr, "%d", val);
+    rc = argvAdd(argvp, valstr);
+    free(valstr);
+    return rc;
+}
+
+int argvAppend(ARGV_t * argvp, ARGV_const_t av)
 {
     ARGV_t argv = *argvp;
     int argc = argvCount(argv);
@@ -175,17 +169,13 @@ int argvAppend(/*@out@*/ ARGV_t * argvp, const ARGV_t av)
 
 int argvSplit(ARGV_t * argvp, const char * str, const char * seps)
 {
-    static char whitespace[] = " \f\n\r\t\v";
-    char * dest = xmalloc(strlen(str) + 1);
+    char *dest = xmalloc(strlen(str) + 1);
     ARGV_t argv;
     int argc = 1;
     const char * s;
     char * t;
     int c;
 
-    if (seps == NULL)
-	seps = whitespace;
-
     for (argc = 1, s = str, t = dest; (c = *s); s++, t++) {
 	if (strchr(seps, c)) {
 	    argc++;
@@ -205,63 +195,18 @@ int argvSplit(ARGV_t * argvp, const char * str, const char * seps)
     }
     argv[c] = NULL;
     *argvp = argv;
-/*@-nullstate@*/
+    free(dest);
     return 0;
-/*@=nullstate@*/
 }
 
-char * argvJoin(ARGV_t argv)
+char *argvJoin(ARGV_const_t argv, const char *sep)
 {
-    size_t nb = 0;
-    int argc;
-    char *t, *te;
+    char *dest = NULL;
+    char * const *arg;
 
-    for (argc = 0; argv[argc] != NULL; argc++) {
-	if (argc != 0)
-	    nb++;
-	nb += strlen(argv[argc]);
-    }
-    nb++;
-
-    te = t = xmalloc(nb);
-    *te = '\0';
-    for (argc = 0; argv[argc] != NULL; argc++) {
-	if (argc != 0)
-	    *te++ = ' ';
-	te = stpcpy(te, argv[argc]);
-    }
-    *te = '\0';
-    return t;
+    for (arg = argv; *arg; arg++) {
+	rstrscat(&dest, *arg, *(arg+1) ? sep : "", NULL);
+    } 
+    return dest;
 }
-
-int argvFgets(ARGV_t * argvp, void * fd)
-{
-    FILE * fp = (fd ? fdGetFILE(fd) : stdin);
-    ARGV_t av = NULL;
-    char buf[BUFSIZ];
-    char * b, * be;
-    int rc = 0;
-
-    if (fp == NULL)
-	return -2;
-    while (!rc && (b = fgets(buf, sizeof(buf), fp)) != NULL) {
-	buf[sizeof(buf)-1] = '\0';
-	be = b + strlen(buf) - 1;
-	while (strchr("\r\n", *be) != NULL)
-	    *be-- = '\0';
-	rc = argvAdd(&av, b);
-    }
-
-    if (!rc)
-	rc = ferror(fp);
-    if (!rc)
-	rc = (feof(fp) ? 0 : 1);
-    if (!rc && argvp)
-	*argvp = av;
-    else
-	av = argvFree(av);
     
-    return rc;
-}
-
-/*@=bounds@*/
