@@ -1,13 +1,6 @@
-/*@-bounds -realcompare -sizeoftype -protoparammatch @*/
 #include "system.h"
 
 #ifdef	WITH_LUA
-#include <rpmio.h>
-#include <rpmmacro.h>
-#include <rpmerr.h>
-#include <rpmurl.h>
-#include <rpmhook.h>
-
 #include <lua.h>
 #include <lualib.h>
 #include <lauxlib.h>
@@ -17,104 +10,81 @@
 #include <unistd.h>
 #include <assert.h>
 
+#include <rpm/rpmio.h>
+#include <rpm/rpmmacro.h>
+#include <rpm/rpmlog.h>
+#include <rpm/rpmurl.h>
+#include <rpm/rpmfileutil.h>
+#include "rpmio/rpmhook.h"
+
 #define _RPMLUA_INTERNAL
-#include "rpmlua.h"
+#include "rpmio/rpmlua.h"
 
 #include "debug.h"
 
-#if !defined(HAVE_VSNPRINTF)
-static inline int vsnprintf(char * buf, /*@unused@*/ int nb,
-			    const char * fmt, va_list ap)
-{
-    return vsprintf(buf, fmt, ap);
-}
-#endif
-
 #define INITSTATE(_lua, lua) \
     rpmlua lua = _lua ? _lua : \
 	    (globalLuaState ? globalLuaState : \
-			/*@-mods@*/ \
+			\
 			(globalLuaState = rpmluaNew()) \
-			/*@=mods@*/ \
+			\
 	    )
 
-/*@only@*/ /*@unchecked@*/ /*@null@*/
 static rpmlua globalLuaState = NULL;
 
-static int luaopen_rpm(lua_State *L)
-	/*@modifies L @*/;
-static int rpm_print(lua_State *L)
-	/*@globals fileSystem @*/
-	/*@modifies L, fileSystem @*/;
+static int luaopen_rpm(lua_State *L);
+static int rpm_print(lua_State *L);
+
+rpmlua rpmluaGetGlobalState(void)
+{
+    INITSTATE(NULL, lua);
+    return lua;
+}
 
 rpmlua rpmluaNew()
 {
     rpmlua lua = (rpmlua) xcalloc(1, sizeof(*lua));
-    lua_State *L = lua_open();
-    /*@-readonlytrans@*/
-    /*@observer@*/ /*@unchecked@*/
-    static const luaL_reg lualibs[] = {
-	{"base", luaopen_base},
-	{"table", luaopen_table},
-	{"io", luaopen_io},
-	{"string", luaopen_string},
-	{"debug", luaopen_debug},
-	{"loadlib", luaopen_loadlib},
+    struct stat st;
+    const luaL_reg *lib;
+    char *initlua = rpmGenPath(rpmConfigDir(), "init.lua", NULL);
+   
+    static const luaL_reg extlibs[] = {
 	{"posix", luaopen_posix},
 	{"rex", luaopen_rex},
 	{"rpm", luaopen_rpm},
 	{NULL, NULL},
     };
-    /*@observer@*/ /*@unchecked@*/
-    const luaL_reg *lib = lualibs;
-    /*@=readonlytrans@*/
-
+    
+    lua_State *L = lua_open();
+    luaL_openlibs(L);
     lua->L = L;
-    for (; lib->name; lib++) {
-/*@-noeffectuncon@*/
-	(void) lib->func(L);
-/*@=noeffectuncon@*/
+
+    for (lib = extlibs; lib->name; lib++) {
+	lua_pushcfunction(L, lib->func);
+	lua_pushstring(L, lib->name);
+	lua_call(L, 1, 0);
 	lua_settop(L, 0);
     }
-#define	_LUADOTDIR	"%{?_usrlibrpm}%{!?_usrlibrpm:" USRLIBRPM "}"
-    {	const char * _lua_path = rpmGetPath(_LUADOTDIR, "/lua/?.lua", NULL);
-	if (_lua_path != NULL) {
-	    lua_pushliteral(L, "LUA_PATH");
-	    lua_pushstring(L, _lua_path);
-	    free(_lua_path);
-	}
-    }
+    lua_pushliteral(L, "LUA_PATH");
+    lua_pushfstring(L, "%s/%s", rpmConfigDir(), "/lua/?.lua");
     lua_rawset(L, LUA_GLOBALSINDEX);
     lua_pushliteral(L, "print");
     lua_pushcfunction(L, rpm_print);
     lua_rawset(L, LUA_GLOBALSINDEX);
     rpmluaSetData(lua, "lua", lua);
-    {	const char * _lua_init = rpmGetPath(_LUADOTDIR, "/init.lua", NULL);
-	if (_lua_init != NULL) {
-	    struct stat st;
-	    if (Stat(_lua_init, &st) != -1)
-		(void)rpmluaRunScriptFile(lua, _lua_init);
-	    free(_lua_init);
-	}
-    }
-#undef	_LUADOTDIR
+    if (stat(initlua, &st) != -1)
+	(void)rpmluaRunScriptFile(lua, initlua);
+    free(initlua);
     return lua;
 }
 
 void *rpmluaFree(rpmlua lua)
-	/*@globals globalLuaState @*/
-	/*@modifies globalLuaState @*/
 {
-/*@-branchstate@*/
-    if (lua == NULL) {
-	lua = globalLuaState;
-	globalLuaState = NULL;
-    }
-/*@=branchstate@*/
     if (lua) {
 	if (lua->L) lua_close(lua->L);
 	free(lua->printbuf);
 	free(lua);
+	if (lua == globalLuaState) globalLuaState = NULL;
     }
     return NULL;
 }
@@ -134,17 +104,14 @@ void rpmluaSetData(rpmlua _lua, const char *key, const void *data)
 }
 
 static void *getdata(lua_State *L, const char *key)
-	/*@modifies L @*/
 {
     void *ret = NULL;
     lua_pushliteral(L, "rpm_");
     lua_pushstring(L, key);
     lua_concat(L, 2);
     lua_rawget(L, LUA_REGISTRYINDEX);
-/*@-branchstate@*/
     if (lua_islightuserdata(L, -1))
 	ret = lua_touserdata(L, -1);
-/*@=branchstate@*/
     lua_pop(L, 1);
     return ret;
 }
@@ -162,6 +129,7 @@ void rpmluaSetPrintBuffer(rpmlua _lua, int flag)
     free(lua->printbuf);
     lua->printbuf = NULL;
     lua->printbufsize = 0;
+    lua->printbufused = 0;
 }
 
 const char *rpmluaGetPrintBuffer(rpmlua _lua)
@@ -171,7 +139,6 @@ const char *rpmluaGetPrintBuffer(rpmlua _lua)
 }
 
 static int pushvar(lua_State *L, rpmluavType type, void *value)
-	/*@modifies L @*/
 {
     int ret = 0;
     switch (type) {
@@ -217,14 +184,11 @@ void rpmluaSetVar(rpmlua _lua, rpmluav var)
 }
 
 static void popvar(lua_State *L, rpmluavType *type, void *value)
-	/*@modifies L, *type, *value @*/
 {
     switch (lua_type(L, -1)) {
     case LUA_TSTRING:
 	*type = RPMLUAV_STRING;
-/*@-observertrans -dependenttrans @*/
 	*((const char **)value) = lua_tostring(L, -1);
-/*@=observertrans =dependenttrans @*/
 	break;
     case LUA_TNUMBER:
 	*type = RPMLUAV_NUMBER;
@@ -262,12 +226,20 @@ void rpmluaGetVar(rpmlua _lua, rpmluav var)
 #define FINDKEY_CREATE 1
 #define FINDKEY_REMOVE 2
 static int findkey(lua_State *L, int oper, const char *key, va_list va)
-	/*@modifies L @*/
 {
-    char buf[BUFSIZ];
+    char *buf;
     const char *s, *e;
     int ret = 0;
-    (void) vsnprintf(buf, BUFSIZ, key, va);
+    int blen;
+
+    blen = vsnprintf(NULL, 0, key, va);
+    if (blen <= 0) {
+	return -1;
+    }
+
+    buf = xmalloc(blen + 1);
+    vsnprintf(buf, blen + 1, key, va);
+
     s = e = buf;
     lua_pushvalue(L, LUA_GLOBALSINDEX);
     for (;;) {
@@ -280,13 +252,12 @@ static int findkey(lua_State *L, int oper, const char *key, va_list va)
 			lua_pushnil(L);
 			lua_rawset(L, -3);
 			lua_pop(L, 1);
-			/*@switchbreak@*/ break;
+			break;
 		    }
-		    /*@fallthrough@*/
 		case FINDKEY_RETURN:
 		    lua_rawget(L, -2);
 		    lua_remove(L, -2);
-		    /*@switchbreak@*/ break;
+		    break;
 		case FINDKEY_CREATE:
 		    lua_rawget(L, -2);
 		    if (!lua_istable(L, -1)) {
@@ -297,7 +268,7 @@ static int findkey(lua_State *L, int oper, const char *key, va_list va)
 			lua_rawset(L, -4);
 		    }
 		    lua_remove(L, -2);
-		    /*@switchbreak@*/ break;
+		    break;
 		}
 	    }
 	    if (*e == '\0')
@@ -311,6 +282,7 @@ static int findkey(lua_State *L, int oper, const char *key, va_list va)
 	}
 	e++;
     }
+    free(buf);
 
     return ret;
 }
@@ -379,7 +351,6 @@ void rpmluavSetListMode(rpmluav var, int flag)
 void rpmluavSetKey(rpmluav var, rpmluavType type, const void *value)
 {
     var->keyType = type;
-/*@-assignexpose -branchstate -temptrans @*/
     switch (type) {
 	case RPMLUAV_NUMBER:
 	    var->key.num = *((double *)value);
@@ -390,13 +361,11 @@ void rpmluavSetKey(rpmluav var, rpmluavType type, const void *value)
 	default:
 	    break;
     }
-/*@=assignexpose =branchstate =temptrans @*/
 }
 
 void rpmluavSetValue(rpmluav var, rpmluavType type, const void *value)
 {
     var->valueType = type;
-/*@-assignexpose -branchstate -temptrans @*/
     switch (type) {
 	case RPMLUAV_NUMBER:
 	    var->value.num = *((const double *)value);
@@ -407,13 +376,11 @@ void rpmluavSetValue(rpmluav var, rpmluavType type, const void *value)
 	default:
 	    break;
     }
-/*@=assignexpose =branchstate =temptrans @*/
 }
 
 void rpmluavGetKey(rpmluav var, rpmluavType *type, void **value)
 {
     *type = var->keyType;
-/*@-onlytrans@*/
     switch (var->keyType) {
 	case RPMLUAV_NUMBER:
 	    *((double **)value) = &var->key.num;
@@ -424,13 +391,11 @@ void rpmluavGetKey(rpmluav var, rpmluavType *type, void **value)
 	default:
 	    break;
     }
-/*@=onlytrans@*/
 }
 
 void rpmluavGetValue(rpmluav var, rpmluavType *type, void **value)
 {
     *type = var->valueType;
-/*@-onlytrans@*/
     switch (var->valueType) {
 	case RPMLUAV_NUMBER:
 	    *((double **)value) = &var->value.num;
@@ -441,7 +406,6 @@ void rpmluavGetValue(rpmluav var, rpmluavType *type, void **value)
 	default:
 	    break;
     }
-/*@=onlytrans@*/
 }
 
 void rpmluavSetKeyNum(rpmluav var, double value)
@@ -489,12 +453,10 @@ int rpmluaCheckScript(rpmlua _lua, const char *script, const char *name)
     INITSTATE(_lua, lua);
     lua_State *L = lua->L;
     int ret = 0;
-/*@-branchstate@*/
     if (name == NULL)
 	name = "<lua>";
-/*@=branchstate@*/
     if (luaL_loadbuffer(L, script, strlen(script), name) != 0) {
-	rpmError(RPMERR_SCRIPT,
+	rpmlog(RPMLOG_ERR,
 		_("invalid syntax in lua scriptlet: %s\n"),
 		  lua_tostring(L, -1));
 	ret = -1;
@@ -508,17 +470,15 @@ int rpmluaRunScript(rpmlua _lua, const char *script, const char *name)
     INITSTATE(_lua, lua);
     lua_State *L = lua->L;
     int ret = 0;
-/*@-branchstate@*/
     if (name == NULL)
 	name = "<lua>";
-/*@=branchstate@*/
     if (luaL_loadbuffer(L, script, strlen(script), name) != 0) {
-	rpmError(RPMERR_SCRIPT, _("invalid syntax in lua script: %s\n"),
+	rpmlog(RPMLOG_ERR, _("invalid syntax in lua script: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	ret = -1;
     } else if (lua_pcall(L, 0, 0, 0) != 0) {
-	rpmError(RPMERR_SCRIPT, _("lua script failed: %s\n"),
+	rpmlog(RPMLOG_ERR, _("lua script failed: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	ret = -1;
@@ -532,12 +492,12 @@ int rpmluaRunScriptFile(rpmlua _lua, const char *filename)
     lua_State *L = lua->L;
     int ret = 0;
     if (luaL_loadfile(L, filename) != 0) {
-	rpmError(RPMERR_SCRIPT, _("invalid syntax in lua file: %s\n"),
+	rpmlog(RPMLOG_ERR, _("invalid syntax in lua file: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	ret = -1;
     } else if (lua_pcall(L, 0, 0, 0) != 0) {
-	rpmError(RPMERR_SCRIPT, _("lua script failed: %s\n"),
+	rpmlog(RPMLOG_ERR, _("lua script failed: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	ret = -1;
@@ -547,8 +507,6 @@ int rpmluaRunScriptFile(rpmlua _lua, const char *filename)
 
 /* From lua.c */
 static int rpmluaReadline(lua_State *L, const char *prompt)
-	/*@globals fileSystem @*/
-	/*@modifies L, fileSystem @*/
 {
    static char buffer[1024];
    if (prompt) {
@@ -565,8 +523,6 @@ static int rpmluaReadline(lua_State *L, const char *prompt)
 
 /* Based on lua.c */
 static void _rpmluaInteractive(lua_State *L)
-	/*@globals fileSystem @*/
-	/*@modifies L, fileSystem @*/
 {
    (void) fputs("\n", stdout);
    printf("RPM Interactive %s Interpreter\n", LUA_VERSION);
@@ -576,25 +532,21 @@ static void _rpmluaInteractive(lua_State *L)
       if (rpmluaReadline(L, "> ") == 0)
 	 break;
       if (lua_tostring(L, -1)[0] == '=') {
-/*@-evalorder@*/
 	 (void) lua_pushfstring(L, "print(%s)", lua_tostring(L, -1)+1);
-/*@=evalorder@*/
 	 lua_remove(L, -2);
       }
       for (;;) {
-/*@-evalorder@*/
 	 rc = luaL_loadbuffer(L, lua_tostring(L, -1),
 			      lua_strlen(L, -1), "<lua>");
-/*@=evalorder@*/
 	 if (rc == LUA_ERRSYNTAX &&
 	     strstr(lua_tostring(L, -1), "near `<eof>'") != NULL) {
 	    if (rpmluaReadline(L, ">> ") == 0)
-	       /*@innerbreak@*/ break;
+	       break;
 	    lua_remove(L, -2); /* Remove error */
 	    lua_concat(L, 2);
-	    /*@innercontinue@*/ continue;
+	    continue;
 	 }
-	 /*@innerbreak@*/ break;
+	 break;
       }
       if (rc == 0)
 	 rc = lua_pcall(L, 0, 0, 0);
@@ -607,20 +559,16 @@ static void _rpmluaInteractive(lua_State *L)
    (void) fputs("\n", stdout);
 }
 
-/*@-mods@*/
 void rpmluaInteractive(rpmlua _lua)
 {
     INITSTATE(_lua, lua);
     _rpmluaInteractive(lua->L);
 }
-/*@=mods@*/
 
 /* ------------------------------------------------------------------ */
 /* Lua API */
 
 static int rpm_expand(lua_State *L)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies L, rpmGlobalMacroContext @*/
 {
     const char *str = luaL_checkstring(L, 1);
     lua_pushstring(L, rpmExpand(str, NULL));
@@ -628,8 +576,6 @@ static int rpm_expand(lua_State *L)
 }
 
 static int rpm_define(lua_State *L)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies L, rpmGlobalMacroContext @*/
 {
     const char *str = luaL_checkstring(L, 1);
     (void) rpmDefineMacro(NULL, str, 0);
@@ -637,22 +583,18 @@ static int rpm_define(lua_State *L)
 }
 
 static int rpm_interactive(lua_State *L)
-	/*@globals fileSystem @*/
-	/*@modifies L, fileSystem @*/
 {
     _rpmluaInteractive(L);
     return 0;
 }
 
 typedef struct rpmluaHookData_s {
-/*@shared@*/
     lua_State *L;
     int funcRef;
     int dataRef;
 } * rpmluaHookData;
 
 static int rpmluaHookWrapper(rpmhookArgs args, void *data)
-    /*@*/
 {
     rpmluaHookData hookdata = (rpmluaHookData)data;
     lua_State *L = hookdata->L;
@@ -665,27 +607,27 @@ static int rpmluaHookWrapper(rpmhookArgs args, void *data)
 	    case 's':
 		lua_pushstring(L, args->argv[i].s);
 		lua_rawseti(L, -2, i+1);
-		/*@switchbreak@*/ break;
+		break;
 	    case 'i':
 		lua_pushnumber(L, (lua_Number)args->argv[i].i);
 		lua_rawseti(L, -2, i+1);
-		/*@switchbreak@*/ break;
+		break;
 	    case 'f':
 		lua_pushnumber(L, (lua_Number)args->argv[i].f);
 		lua_rawseti(L, -2, i+1);
-		/*@switchbreak@*/ break;
+		break;
 	    case 'p':
 		lua_pushlightuserdata(L, args->argv[i].p);
 		lua_rawseti(L, -2, i+1);
-		/*@switchbreak@*/ break;
+		break;
 	    default:
                 (void) luaL_error(L, "unsupported type '%c' as "
                               "a hook argument\n", args->argt[i]);
-		/*@switchbreak@*/ break;
+		break;
 	}
     }
     if (lua_pcall(L, 1, 1, 0) != 0) {
-	rpmError(RPMERR_SCRIPT, _("lua hook failed: %s\n"),
+	rpmlog(RPMLOG_ERR, _("lua hook failed: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
     } else {
@@ -697,8 +639,6 @@ static int rpmluaHookWrapper(rpmhookArgs args, void *data)
 }
 
 static int rpm_register(lua_State *L)
-	/*@globals internalState @*/
-	/*@modifies L, internalState @*/
 {
     if (!lua_isstring(L, 1)) {
 	(void) luaL_argerror(L, 1, "hook name expected");
@@ -711,9 +651,7 @@ static int rpm_register(lua_State *L)
 	hookdata->dataRef = luaL_ref(L, LUA_REGISTRYINDEX);
 	lua_pushvalue(L, 2);
 	hookdata->funcRef = luaL_ref(L, LUA_REGISTRYINDEX);
-/*@-temptrans@*/
 	hookdata->L = L;
-/*@=temptrans@*/
 	rpmhookRegister(lua_tostring(L, 1), rpmluaHookWrapper, hookdata);
 	return 1;
     }
@@ -721,7 +659,6 @@ static int rpm_register(lua_State *L)
 }
 
 static int rpm_unregister(lua_State *L)
-	/*@modifies L @*/
 {
     if (!lua_isstring(L, 1)) {
 	(void) luaL_argerror(L, 1, "hook name expected");
@@ -737,8 +674,6 @@ static int rpm_unregister(lua_State *L)
 }
 
 static int rpm_call(lua_State *L)
-	/*@globals internalState @*/
-	/*@modifies L, internalState @*/
 {
     if (!lua_isstring(L, 1)) {
 	(void) luaL_argerror(L, 1, "hook name expected");
@@ -752,10 +687,9 @@ static int rpm_call(lua_State *L)
 		case LUA_TNIL:
 		    argt[i] = 'p';
 		    args->argv[i].p = NULL;
-		    /*@switchbreak@*/ break;
+		    break;
 		case LUA_TNUMBER: {
 		    float f = (float)lua_tonumber(L, i+1);
-/*@+relaxtypes@*/
 		    if (f == (int)f) {
 			argt[i] = 'i';
 			args->argv[i].i = (int)f;
@@ -763,38 +697,33 @@ static int rpm_call(lua_State *L)
 			argt[i] = 'f';
 			args->argv[i].f = f;
 		    }
-/*@=relaxtypes@*/
-		}   /*@switchbreak@*/ break;
+		}   break;
 		case LUA_TSTRING:
 		    argt[i] = 's';
 		    args->argv[i].s = lua_tostring(L, i+1);
-		    /*@switchbreak@*/ break;
+		    break;
 		case LUA_TUSERDATA:
 		case LUA_TLIGHTUSERDATA:
 		    argt[i] = 'p';
 		    args->argv[i].p = lua_touserdata(L, i+1);
-		    /*@switchbreak@*/ break;
+		    break;
 		default:
 		    (void) luaL_error(L, "unsupported Lua type passed to hook");
 		    argt[i] = 'p';
 		    args->argv[i].p = NULL;
-		    /*@switchbreak@*/ break;
+		    break;
 	    }
 	}
-/*@-compdef -kepttrans -usereleased @*/
 	args->argt = argt;
 	rpmhookCallArgs(name, args);
 	free(argt);
 	(void) rpmhookArgsFree(args);
-/*@=compdef =kepttrans =usereleased @*/
     }
     return 0;
 }
 
 /* Based on luaB_print. */
 static int rpm_print (lua_State *L)
-	/*@globals fileSystem @*/
-	/*@modifies L, fileSystem @*/
 {
     rpmlua lua = (rpmlua)getdata(L, "lua");
     int n = lua_gettop(L);  /* number of arguments */
@@ -826,22 +755,18 @@ static int rpm_print (lua_State *L)
 	}
 	lua_pop(L, 1);  /* pop result */
     }
-    lua_pop(L, 1);
     if (!lua->storeprint) {
 	(void) fputs("\n", stdout);
     } else {
-	if (lua->printbufused+1 >= lua->printbufsize) {
+	if (lua->printbufused+1 > lua->printbufsize) {
 	    lua->printbufsize += 512;
 	    lua->printbuf = xrealloc(lua->printbuf, lua->printbufsize);
 	}
-	lua->printbuf[lua->printbufused++] = '\n';
 	lua->printbuf[lua->printbufused] = '\0';
     }
     return 0;
 }
 
-/*@-readonlytrans@*/
-/*@observer@*/ /*@unchecked@*/
 static const luaL_reg rpmlib[] = {
     {"expand", rpm_expand},
     {"define", rpm_define},
@@ -851,10 +776,8 @@ static const luaL_reg rpmlib[] = {
     {"interactive", rpm_interactive},
     {NULL, NULL}
 };
-/*@=readonlytrans@*/
 
 static int luaopen_rpm(lua_State *L)
-	/*@modifies L @*/
 {
     lua_pushvalue(L, LUA_GLOBALSINDEX);
     luaL_openlib(L, "rpm", rpmlib, 0);
@@ -862,4 +785,3 @@ static int luaopen_rpm(lua_State *L)
 }
 #endif	/* WITH_LUA */
 
-/*@=bounds =realcompare =sizeoftype =protoparammatch @*/
